==> ec2init-0.11-py3-none-any.whl: ec2init/plans/plan_utils.py <==
import configparser
import pathlib
import base64
import json
from configparser import ConfigParser
import logging
import re
import os

from typing import Dict, List, Match, Optional

from ec2init.plans.init.callback_hopsworks import CallbackHopsworks
from ec2init.plans.init.start_services import StartServices

from ec2init.utils.system import SystemUtils
from ec2init.utils.cloud import get_cloud_provider

REPORT_ENDPOINT = (
    "POST", "{gateway}/eventsv2/instance-state/users/{uid}/instances/{instance_id}")

logger = logging.getLogger(__name__)

class PlanUtils:
    def __init__(self, checks_dir: pathlib.Path) -> None:
        self.checks_dir = checks_dir

    def get_procedure_check_file(self, procedure_name: str) -> pathlib.Path:
        return self.checks_dir / procedure_name

    def check_file_exists(self, check_file: pathlib.Path) -> bool:
        return check_file.is_file()

    def remove_check_file(self, check_file: pathlib.Path) -> None:
        if self.check_file_exists(check_file):
            check_file.unlink()
        
    def get_procedures_check_directory(self) -> pathlib.Path:
        return self.checks_dir

    @staticmethod
    def create_checks_directory(config: ConfigParser) -> pathlib.Path:
        installation_dir = config.get('general', 'installation_dir', fallback='/root/ec2init')
        ec2init_dir = pathlib.Path(installation_dir, "ec2init_checks")

        if not ec2init_dir.is_dir():
            ec2init_dir.mkdir()
        return ec2init_dir

    @staticmethod
    def revoke_old_certificates(config: ConfigParser, certs_dir: str, certs_to_revoke: List[str] = ["L=consul", "L=elastic", "L=elkadmin", "L=flink", "L=glassfish", "L=hdfs",
                                                                                                          "L=hive", "L=hopsmon", "L=kafka", "L=kagent", "L=livy", "L=mapred", "L=mysql", "L=rmyarn", "L=yarn", "L=onlinefs"]) -> None:
        try:
            intermediate_certs_dir = f"{certs_dir}/intermediate"
            openssl_intermediate_config = f"{intermediate_certs_dir}/openssl-intermediate.cnf"
            openssl_db = f"{intermediate_certs_dir}/index.txt"
            
            user_cert_pattern = re.compile("^/C=SE/ST=Sweden/L=(\w+)/OU=(\d)/CN=(.*)$")
            glassfish_internal_pattern = re.compile("^/OU=(\d)/CN=(.*)$")

            if not os.path.isfile(openssl_db):
                #there is no old certificates 
                return
            # Load all valid certs in openssl_db
            openssl_certs: List[str] = []
            with open(openssl_db, "r") as file:
                openssl_certs = file.readlines()
            valid_certs = [l.strip().split('\t') for l in openssl_certs if l.startswith('V')]

            # get the master password 
            installation_dir = config.get('general', 'installation_dir', fallback='/root/ec2init')
            process = SystemUtils.execute_command(["grep", "MASTER_PASSWORD=", f"{installation_dir}/generate_glassfish_internal_x509.sh"])
            master_password = process.stdout.decode("utf-8").replace("MASTER_PASSWORD=","").strip()

            # loop over valid_certs and revoke old ones and delete their associated files
            for _, _, _, serial, _, dn_name in valid_certs:
                cert_filter = [cert for cert in certs_to_revoke if cert in dn_name]
                if len(cert_filter) > 0:
                    logger.info("Revoke certificate for %s", dn_name)
                    cert_file = f"{intermediate_certs_dir}/newcerts/{serial}.pem"
                    SystemUtils.execute_command(
                        ["openssl", "ca", "-config", openssl_intermediate_config, "-revoke", cert_file, "-key", master_password])
                    name_parts = user_cert_pattern.match(dn_name)
                    if name_parts:
                        # CN__L__OU.cert.pem
                        cert_file_name = f"{intermediate_certs_dir}/certs/{name_parts.group(3)}__{name_parts.group(1)}__{name_parts.group(2)}.cert.pem"
                        logger.info("Removing revoked cert file %s", cert_file_name)
                        os.remove(cert_file_name)
                    elif glassfish_internal_pattern.match(dn_name):
                        match = glassfish_internal_pattern.match(dn_name)
                        assert match is not None
                        gf_cert_file = pathlib.Path(intermediate_certs_dir, "certs", f"{match.group(2)}__glassfish__{match.group(1)}.cert.pem")
                        logger.info("Removing revoked certificate %s", gf_cert_file)
                        os.remove(str(gf_cert_file))
        except Exception as ex:
            logger.info("failed to revoke old certificates: %s", ex)
            raise ex



class HopsPlanUtils(PlanUtils):

    def __init__(self, config: ConfigParser, user_password: Optional[str], **kwargs: str) -> None:
        super().__init__(PlanUtils.create_checks_directory(config))
        self.config = config
        self.gateway = kwargs.get("gateway", "gateway")
        self.instance_id = kwargs.get("instance_id", "instance_id")
        self.uid = kwargs.get("uid", "uid")
        self.user_email = kwargs.get("user_email", "user_email")
        self.user_password = user_password
        self.api_key = kwargs.get("api_key", "")
        self.node_id = kwargs.get("node_id", "node_id")
        self.cloud_provider = kwargs.get("cloud_provider", "cloud_provider")
        self.issue_lets_encrypt = True if kwargs.get("issue_lets_encrypt", "true").lower() == "true" else False
        self.has_public_ip = True if kwargs.get("has_public_ip", "true").lower() == "true" else False
        self.user_mgm_type = kwargs.get("user_management_type", "")
        self.cluster_domain_prefix = kwargs.get("cluster_domain_prefix", "")

    def base64_to_json(self, conf: str) -> Dict[str, str]:
        if conf:
            decoded = base64.b64decode(conf)
            return json.loads(decoded)
        return {}

    def get_cloud_tmp_test_file(self) -> pathlib.Path:
        path = self.checks_dir / "test-obj"
        f = open(path, 'w+')
        f.write("S3/Azure test file")
        f.close()
        return path

    def get_cloud_versioning_enabled_file(self) -> pathlib.Path:
        return self.checks_dir / "versioning-enabled" 

    def create_callback_procedure(self, state: str, stage: str = None, check_file: pathlib.Path = None, instance_type: str = "") -> CallbackHopsworks:
        return CallbackHopsworks(CallbackHopsworks.__name__, self.config, self.gateway, self.instance_id,  self.node_id,
                                 instance_type, state, self.uid, REPORT_ENDPOINT, self.has_public_ip, self.user_email, self.user_password, stage, check_file, self.api_key, 
                                 self.cloud_provider, user_mgm_type=self.user_mgm_type)

    def has_initialized_hops_ca(self) -> bool:
        ca_dir = self.config.get(
            'general', 'certs_dir', fallback="/srv/hops/certs-dir")
        # This file is deleted when baking the image
        ca_private_key = pathlib.Path(ca_dir, "private", "ca.key.pem")
        return ca_private_key.is_file()

    def has_issued_certificate(self) -> bool:
        # Mock it if it's unmanaged instance or instance has no internet connectivity and issue
        mock_msg = "Mock has_issued_certificate check because"
        if self.config.getboolean('general', 'unmanaged', fallback=False):
            logger.warning(f"{mock_msg} it is an unmanaged instance")
            return True
        if not self.issue_lets_encrypt:
            logger.warning(f"{mock_msg} flag issue_lets_encrypt is set to False")
            return True
        domain_name = get_cloud_provider(
            self.cloud_provider).get_public_domain_name(self.config, self.instance_id, self.cluster_domain_prefix)
        certificate = pathlib.Path(
            '/etc', 'letsencrypt', 'live', domain_name, 'cert.pem')
        return certificate.is_file()

    def has_configured_glassfish(self) -> bool:
        if self.config.getboolean('general', 'unmanaged', fallback=False)\
            or not self.issue_lets_encrypt:
            return StartServices.is_service_enabled('glassfish-domain1') \
                and StartServices._is_service_running('glassfish-domain1')
        lets_encrypt_dir = self.config.get(
            'initplan', 'lets_encrypt_dir', fallback="/etc/letsencrypt")
        script = pathlib.Path(lets_encrypt_dir, 'renewal-hooks',
                              'deploy', 'deploy2glassfish_hook.sh')
        return script.is_file()

    def has_started_enabled_consul(self) -> bool:
        return StartServices._is_service_running("consul") and StartServices.is_service_enabled("consul")

    def has_started_enabled_kagent(self) -> bool:
        return StartServices._is_service_running("kagent") and StartServices.is_service_enabled("kagent")

    def has_enabled_ec2update_service(self) -> bool:
        if self.config.getboolean('general', 'unmanaged', fallback=False):
            return True
        return StartServices.is_service_enabled("ec2update")

    def has_started_enabled_cloudwatch_agent(self) -> bool:
        return StartServices._is_service_running("amazon-cloudwatch-agent")\
            and StartServices.is_service_enabled("amazon-cloudwatch-agent")
            
    @staticmethod
    def clean_hopsworks_version(version: str) -> str:
        return version.replace('-SNAPSHOT', '')

==> ec2init-0.17-py3-none-any.whl: ec2init/plans/plan_utils.py <==
import configparser
import pathlib
import base64
import json
from configparser import ConfigParser
import logging
import re
import os

from typing import Any, Dict, List, Optional

from ec2init.plans.init.callback_hopsworks import CallbackHopsworks
from ec2init.plans.init.start_services import StartServices
from ec2init.utils import cloud

from ec2init.utils.system import SystemUtils
from ec2init.utils.cloud import get_cloud_provider
from ec2init.utils.http import HttpUtils

REPORT_ENDPOINT = (
    "POST", "{gateway}/eventsv2/instance-state/users/{uid}/instances/{instance_id}")

CLUSTER_STATUS_ENDPOINT = (
    "GET", "{gateway}/eventsv2/instance-state/users/{uid}/instances/{instance_id}")

logger = logging.getLogger(__name__)

class PlanUtils:
    def __init__(self, checks_dir: pathlib.Path) -> None:
        self.checks_dir = checks_dir

    def get_procedure_check_file(self, procedure_name: str) -> pathlib.Path:
        return self.checks_dir / procedure_name

    def check_file_exists(self, check_file: pathlib.Path) -> bool:
        return check_file.is_file()

    def remove_check_file(self, check_file: pathlib.Path) -> None:
        if self.check_file_exists(check_file):
            check_file.unlink()
        
    def get_procedures_check_directory(self) -> pathlib.Path:
        return self.checks_dir

    @staticmethod
    def create_checks_directory(config: ConfigParser) -> pathlib.Path:
        installation_dir = config.get('general', 'installation_dir', fallback='/root/ec2init')
        ec2init_dir = pathlib.Path(installation_dir, "ec2init_checks")

        if not ec2init_dir.is_dir():
            ec2init_dir.mkdir()
        return ec2init_dir

    @staticmethod
    def report_error(plan_name :str, config: configparser.ConfigParser, gateway: str, user_id: str, instance_id: str, api_key :str, error_message: str, node_id: Optional[str]=None, instance_type: Optional[str]=None) -> None:
        method, url = REPORT_ENDPOINT
        url = url.format(gateway=gateway, uid=user_id,
                         instance_id=instance_id)
        data = {"instanceState": "error", "error": error_message}
        if node_id:
            data['nodeId'] = node_id
        if instance_type:
            data['instanceType'] = instance_type
        logger.debug(
            "%s - Reporting an unrecoverable error to %s", plan_name, url)
        logger.debug("%s - Error is %s", plan_name, data)
        headers: Dict[str, str] = {}
        if api_key:
            headers = {"x-api-key": api_key}
        try:
            HttpUtils.maybe_api_call(config, url, method, data=data, headers=headers)
            logger.debug("%s - Reported error", plan_name)
        except Exception as ex:
            logger.error(
                "%s - Could not report error to Hopsworks Cloud: %s", plan_name, ex)

    @staticmethod
    def get_cluster_status(plan_name: str, config: configparser.ConfigParser, gateway: str, user_id: str, instance_id: str, api_key :str) -> Dict[str, Any]:
        method, url = CLUSTER_STATUS_ENDPOINT
        url = url.format(gateway=gateway, uid=user_id, instance_id=instance_id)
        headers: Dict[str, str] = {}
        if api_key:
            headers = {"x-api-key": api_key}
        try:
            logger.info("%s - get cluster status", plan_name)
            return HttpUtils.do_api_call(config, url, method, headers=headers)
        except Exception as ex:
            logger.error(
                "%s - Could not get cluster status from Hopsworks Cloud: %s", plan_name, ex)
        return {}

    @staticmethod
    def get_secondary_node_id(plan_name: str, config: configparser.ConfigParser, gateway: str, user_id: str, instance_id: str, api_key :str, cloud_provider :str) -> str:
        local_ip = cloud.get_cloud_provider(cloud_provider).get_private_ip(config)
        cluster_status = PlanUtils.get_cluster_status(plan_name, config, gateway, user_id, instance_id, api_key)
        secondary_node_id_arr = [node["secondaryNodeId"] for node in cluster_status["payload"] if node["type"] == "secondary" and node['ip'] == local_ip]
        my_node_id = "0"
        if len(secondary_node_id_arr) > 0:
            my_node_id = str(secondary_node_id_arr[0])
        return my_node_id
    
class HopsPlanUtils(PlanUtils):

    def __init__(self, config: ConfigParser, user_password: Optional[str], **kwargs: str) -> None:
        super().__init__(PlanUtils.create_checks_directory(config))
        self.config = config
        self.gateway = kwargs.get("gateway", "gateway")
        self.instance_id = kwargs.get("instance_id", "instance_id")
        self.uid = kwargs.get("uid", "uid")
        self.user_email = kwargs.get("user_email", "user_email")
        self.user_password = user_password
        self.api_key = kwargs.get("api_key", "")
        self.node_id = kwargs.get("node_id", "node_id")
        self.cloud_provider = kwargs.get("cloud_provider", "cloud_provider")
        self.issue_lets_encrypt = True if kwargs.get("issue_lets_encrypt", "true").lower() == "true" else False
        self.has_public_ip = True if kwargs.get("has_public_ip", "true").lower() == "true" else False
        self.user_mgm_type = kwargs.get("user_management_type", "")
        self.cluster_domain_prefix = kwargs.get("cluster_domain_prefix", "")
        # Default is true if not set. create_pwd_user will only be set for OAuth
        self.create_pwd_user = False if kwargs.get("create_pwd_user", "").lower() == "false" else True

    def base64_to_json(self, conf: str) -> Dict[str, str]:
        if conf:
            decoded = base64.b64decode(conf)
            return json.loads(decoded)
        return {}

    def get_cloud_tmp_test_file(self) -> pathlib.Path:
        path = self.checks_dir / "test-obj"
        f = open(path, 'w+')
        f.write("S3/Azure test file")
        f.close()
        return path

    def get_cloud_versioning_enabled_file(self) -> pathlib.Path:
        return self.checks_dir / "versioning-enabled" 

    def create_callback_procedure(self, state: str, stage: str = None, check_file: pathlib.Path = None, instance_type: str = "") -> CallbackHopsworks:
        return CallbackHopsworks(CallbackHopsworks.__name__, self.config, self.gateway, self.instance_id,  self.node_id,
                                 instance_type, state, self.uid, REPORT_ENDPOINT, self.has_public_ip, self.user_email, self.user_password, stage, check_file, self.api_key, 
                                 self.cloud_provider, user_mgm_type=self.user_mgm_type, create_pwd_user=self.create_pwd_user)

    def has_issued_certificate(self) -> bool:
        # Mock it if it's unmanaged instance or instance has no internet connectivity and issue
        mock_msg = "Mock has_issued_certificate check because"
        if self.config.getboolean('general', 'unmanaged', fallback=False):
            logger.warning(f"{mock_msg} it is an unmanaged instance")
            return True
        if not self.issue_lets_encrypt:
            logger.warning(f"{mock_msg} flag issue_lets_encrypt is set to False")
            return True
        domain_name = get_cloud_provider(
            self.cloud_provider).get_public_domain_name(self.config, self.instance_id, self.cluster_domain_prefix)
        certificate = pathlib.Path(
            '/etc', 'letsencrypt', 'live', domain_name, 'cert.pem')
        return certificate.is_file()

    def has_configured_glassfish(self) -> bool:
        if self.config.getboolean('general', 'unmanaged', fallback=False)\
            or not self.issue_lets_encrypt:
            return StartServices.is_service_enabled('glassfish-domain1') \
                and StartServices._is_service_running('glassfish-domain1')
        lets_encrypt_dir = self.config.get(
            'initplan', 'lets_encrypt_dir', fallback="/etc/letsencrypt")
        script = pathlib.Path(lets_encrypt_dir, 'renewal-hooks',
                              'deploy', 'deploy2glassfish_hook.sh')
        return script.is_file()

    def has_started_enabled_consul(self) -> bool:
        return StartServices._is_service_running("consul") and StartServices.is_service_enabled("consul")

    def has_started_enabled_kagent(self) -> bool:
        return StartServices._is_service_running("kagent") and StartServices.is_service_enabled("kagent")

    def has_enabled_ec2update_service(self) -> bool:
        if self.config.getboolean('general', 'unmanaged', fallback=False):
            return True
        return StartServices.is_service_enabled("ec2update")

    def has_started_enabled_cloudwatch_agent(self) -> bool:
        return StartServices._is_service_running("amazon-cloudwatch-agent")\
            and StartServices.is_service_enabled("amazon-cloudwatch-agent")
            
    @staticmethod
    def clean_hopsworks_version(version: str) -> str:
        return version.replace('-SNAPSHOT', '')
